@@ -6,6 +6,7 @@ from shutil import copyfile
 from typing import Dict, Tuple, Callable, Optional, Any, Union, List
 from typing import Iterator, Mapping
 
+import joblib
 import numpy as np
 import torch
 from catalyst.dl import SupervisedRunner, EarlyStoppingCallback, InferCallback, State, Callback
@@ -17,7 +18,7 @@ from torch.optim.optimizer import Optimizer
 from torch.utils.data import DataLoader, Dataset
 
 from dnn_cool.catalyst_utils import InterpretationCallback, TensorboardConverters, ReplaceGatherCallback, \
-    TensorboardConverter
+    TensorboardConverter, to_dict_of_lists, load_inference_results_from_directory
 from dnn_cool.tasks.task_flow import TaskFlow
 from dnn_cool.tasks.development.task_flow import TaskFlowForDevelopment
 from dnn_cool.utils.torch import TransformedSubset, load_model_from_export
@@ -61,12 +62,20 @@ class TrainingArguments(Mapping):
 
 class InferDictCallback(InferCallback):
 
-    def __init__(self, out_key='logits', loaders_to_skip=(), *args, **kwargs):
+    def __init__(self, infer_logdir: Optional[Path] = None,
+                 out_key='logits',
+                 loaders_to_skip=(),
+                 task_flow=None,
+                 *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.loaders_to_skip = loaders_to_skip
         self.out_key = out_key
         self.predictions = {}
         self.targets = {}
+        self.infer_logdir = infer_logdir
+        self.task_flow = task_flow
+        if self.infer_logdir is not None:
+            self.infer_logdir.mkdir(exist_ok=True)
         self.__current_store = None
 
     def on_loader_start(self, state: State):
@@ -108,26 +117,27 @@ class InferDictCallback(InferCallback):
                 self.targets[loader_name][key].append(value.detach().cpu().numpy())
 
     def on_loader_end(self, state: State):
-        self.predictions[state.loader_key] = {
-            key: np.concatenate(value, axis=0)
-            for key, value in self.predictions[state.loader_key].items()
-        }
-
-        self.targets[state.loader_key] = {
-            key: np.concatenate(value, axis=0)
-            for key, value in self.targets[state.loader_key].items()
-        }
-
-
-def load_inference_results_from_directory(logdir):
-    out_dir = logdir / 'infer'
-    out_dir.mkdir(exist_ok=True)
-    res = {}
-    for key in ['logits', 'targets', 'interpretations']:
-        file_path = out_dir / f'{key}.pkl'
-        if file_path.exists():
-            res[key] = torch.load(out_dir / f'{key}.pkl')
-    return res
+        nested_dict = self.predictions[state.loader_key]
+        self.predictions[state.loader_key] = to_dict_of_lists(nested_dict, nested_dict,
+                                                              self.infer_logdir, 'logits', state.loader_key)
+        self.targets[state.loader_key] = to_dict_of_lists(self.targets[state.loader_key], nested_dict,
+                                                          self.infer_logdir, 'targets', state.loader_key)
+        if self.task_flow is None:
+            return
+        for task_name, logits in self.predictions[state.loader_key].items():
+            if task_name.startswith('indices|'):
+                continue
+            task_for_development = self.task_flow.get(task_name)
+            loss = task_for_development.get_per_sample_criterion().loss
+            targets = torch.tensor(self.targets[state.loader_key][task_name])
+            logits = torch.tensor(logits)
+            unreduced = loss(logits, targets).detach().cpu().numpy()
+            out_dir = self.infer_logdir / state.loader_key / 'unrolled_loss'
+            out_dir.mkdir(exist_ok=True)
+            # Since the precondition has been applied already, we can make the assumption
+            # that there are exactly 2 axes, one batch and one for the logits.
+            reduced = unreduced.mean(axis=-1)
+            joblib.dump(reduced, out_dir / f'{task_name}.pkl')
 
 
 class DnnCoolRunnerView:
@@ -198,19 +208,11 @@ class DnnCoolRunnerView:
         loader_idx = ['infer', 'test', 'valid'].index(loader_name)
         global_loader_indices = self.train_test_val_indices[loader_idx]
         mask = loss_local_indices >= 0
-        logits = self.inference_results['logits'][loader_name][task_name][loss_local_indices[mask]]
         task = self.full_flow.get_all_children()[task_name]
-        logits = torch.tensor(logits)
-        activated = task.get_activation()(logits) if task.get_activation() is not None else logits
-        decoded = task.get_decoder()(activated) if task.get_decoder() is not None else activated
         return {
             'global_idx': global_loader_indices[loss_local_indices[mask]],
             'loss_values': loss_values[mask],
-            'targets': self.inference_results['targets'][loader_name][task_name][loss_local_indices[mask]],
-            'activated': activated,
-            'decoded': decoded.numpy(),
-            'logits': logits.numpy(),
-            'task': task
+            'task': task.get_name()
         }
 
     def worst_examples(self, loader_name, task_name, n):
@@ -222,8 +224,10 @@ class DnnCoolRunnerView:
     def extremal_examples(self, loader_name, task_name, n, mult):
         res = self.summarize_loss_values(loader_name, task_name)
         sorter = (mult * res['loss_values']).argsort()
+        res['global_idx_sorted'] = res['global_idx'][sorter]
         top_n_idx = res['global_idx'][sorter[:n]]
-        return top_n_idx, res
+        res[f'global_idx_{n}'] = top_n_idx
+        return res
 
 
 def batch_to_device(batch, device) -> Mapping[str, torch.Tensor]:
@@ -287,7 +291,7 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         if 'loaders' not in kwargs:
             datasets, kwargs['loaders'] = self.get_default_loaders()
 
-        default_callbacks = [self.create_interpretation_callback(**kwargs)] + self.default_callbacks
+        default_callbacks = self.default_callbacks
         kwargs['callbacks'] = kwargs.get('callbacks', default_callbacks)
         super().train(*args, **kwargs)
 
@@ -298,8 +302,10 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
 
         logdir = self.project_dir / Path(kwargs.get('logdir', self.default_logdir))
         kwargs['logdir'] = logdir
-        interpretation_callback = self.create_interpretation_callback(**kwargs)
-        infer_dict_callback = InferDictCallback()
+        store = kwargs.pop('store', True)
+        infer_logdir = logdir / 'infer' if store else None
+        interpretation_callback = self.create_interpretation_callback(infer_logdir=infer_logdir, **kwargs)
+        infer_dict_callback = InferDictCallback(infer_logdir=infer_logdir, task_flow=self.task_flow)
         default_callbacks = OrderedDict([("interpretation", interpretation_callback),
                                          ("inference", infer_dict_callback)])
         if self.balance_dataparallel_memory:
@@ -307,25 +313,11 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
             default_callbacks["dataparallel_reducer"] = replace_gather_callback
         kwargs['callbacks'] = kwargs.get('callbacks', default_callbacks)
         kwargs['model'] = kwargs.get('model', self.model)
-        store = kwargs.pop('store', True)
         kwargs.pop('loader_names_to_skip_in_interpretation', ())
         del kwargs['datasets']
         super().infer(*args, **kwargs)
-        res = {}
-        if 'inference' in kwargs['callbacks']:
-            res['logits'] = kwargs['callbacks']['inference'].predictions
-            res['targets'] = kwargs['callbacks']['inference'].targets
-        if 'interpretation' in kwargs['callbacks']:
-            res['interpretations'] = kwargs['callbacks']['interpretation'].interpretations
 
-        if store:
-            out_dir = logdir / 'infer'
-            out_dir.mkdir(exist_ok=True)
-            for key in res:
-                torch.save(res[key], out_dir / f'{key}.pkl', pickle_protocol=4)
-        return res
-
-    def create_interpretation_callback(self, **kwargs) -> InterpretationCallback:
+    def create_interpretation_callback(self, infer_logdir, **kwargs) -> InterpretationCallback:
         tensorboard_converters = TensorboardConverters(
             logdir=kwargs['logdir'],
             tensorboard_loggers=self.tensor_loggers,
@@ -333,8 +325,9 @@ class DnnCoolSupervisedRunner(SupervisedRunner):
         )
         loaders_to_skip = kwargs.get('loader_names_to_skip_in_interpretation', ())
         interpretation_callback = InterpretationCallback(self.task_flow.get_per_sample_criterion(),
-                                                         tensorboard_converters,
-                                                         loaders_to_skip)
+                                                         infer_logdir=infer_logdir,
+                                                         tensorboard_converters=tensorboard_converters,
+                                                         loaders_to_skip=loaders_to_skip)
         return interpretation_callback
 
     def get_default_loaders(self, shuffle_train=True,
